@@ -25,8 +25,9 @@ import re
 import StringIO
 from MoinMoin import config, wikiutil
 from MoinMoin.macro import Macro
-from MoinMoin.support.python_compatibility import rsplit # Needed for python 2.3
+from MoinMoin import config
 from _creole import Parser as CreoleParser
+from _creole import Rules as CreoleRules
 
 Dependencies = []
 
@@ -38,6 +39,7 @@ class Parser:
     MoinMoin current API.
     """
 
+    extensions = ['.creole']
     # Enable caching
     caching = 1
     Dependencies = Dependencies
@@ -59,15 +61,18 @@ class Parser:
         self.request = request
         self.form = request.form
         self.raw = raw
+        self.rules = MoinRules(wiki_words=True,
+                               url_protocols=config.url_schemas)
 
     def format(self, formatter):
         """Create and call the true parser and emitter."""
 
-        document = CreoleParser(self.raw).parse()
-        result = Emitter(document, formatter, self.request, Macro(self)).emit()
+        document = CreoleParser(self.raw, self.rules).parse()
+        result = Emitter(document, formatter, self.request, Macro(self),
+                         self.rules).emit()
         self.request.write(result)
 
-class Rules:
+class MoinRules(CreoleRules):
     # For the link targets:
     proto = r'http|https|ftp|nntp|news|mailto|telnet|file|irc'
     extern = r'(?P<extern_addr>(?P<extern_proto>%s):.*)' % proto
@@ -81,6 +86,12 @@ class Rules:
         '''
     page = r'(?P<page_name> .* )'
 
+    def __init__(self, *args, **kwargs):
+        CreoleRules.__init__(self, *args, **kwargs)
+        # for addresses
+        self.addr_re = re.compile('|'.join([self.extern, self.attach,
+                                            self.interwiki, self.page]),
+                                  re.X | re.U)
 
 class Emitter:
     """
@@ -88,19 +99,13 @@ class Emitter:
     tree consisting of DocNodes.
     """
 
-    addr_re = re.compile('|'.join([
-            Rules.extern,
-            Rules.attach,
-            Rules.interwiki,
-            Rules.page
-        ]), re.X | re.U) # for addresses
-
-    def __init__(self, root, formatter, request, macro):
+    def __init__(self, root, formatter, request, macro, rules):
         self.root = root
         self.formatter = formatter
         self.request = request
         self.form = request.form
         self.macro = macro
+        self.rules = rules
 
     def get_text(self, node):
         """Try to emit whatever text is in the node."""
@@ -228,14 +233,10 @@ class Emitter:
 #        return self.formatter.smiley(node.content)
 
     def header_emit(self, node):
-        from MoinMoin.support.python_compatibility import hash_new
-
-        pntt = '%s%s%d' % (self.formatter.page.page_name,
-            self.get_text(node), node.level)
-        ident = "head-%s" % hash_new('sha1', pntt.encode(config.charset)).hexdigest()
+        text = self.get_text(node)
         return ''.join([
-            self.formatter.heading(1, node.level, id=ident),
-            self.formatter.text(node.content or ''),
+            self.formatter.heading(1, node.level, id=text),
+            self.formatter.text(text),
             self.formatter.heading(0, node.level),
         ])
 
@@ -263,7 +264,7 @@ class Emitter:
 
     def link_emit(self, node):
         target = node.content
-        m = self.addr_re.match(target)
+        m = self.rules.addr_re.match(target)
         if m:
             if m.group('page_name'):
                 # link to a page
@@ -273,11 +274,7 @@ class Emitter:
                 elif word.startswith(wikiutil.CHILD_PREFIX):
                     word = "%s/%s" % (self.formatter.page.page_name,
                         word[wikiutil.CHILD_PREFIX_LEN:])
-                # handle anchors
-                parts = rsplit(word, "#", 1)
-                anchor = ""
-                if len(parts) == 2:
-                    word, anchor = parts
+                word, anchor = wikiutil.split_anchor(word)
                 return ''.join([
                     self.formatter.pagelink(1, word, anchor=anchor),
                     self.emit_children(node) or self.formatter.text(target),
@@ -296,8 +293,9 @@ class Emitter:
                 # interwiki link
                 wiki = m.group('inter_wiki')
                 page = m.group('inter_page')
+                page, anchor = wikiutil.split_anchor(page)
                 return ''.join([
-                    self.formatter.interwikilink(1, wiki, page),
+                    self.formatter.interwikilink(1, wiki, page, anchor=anchor),
                     self.emit_children(node) or self.formatter.text(page),
                     self.formatter.interwikilink(0),
                 ])
@@ -305,7 +303,7 @@ class Emitter:
                 # link to an attachment
                 scheme = m.group('attach_scheme')
                 attachment = m.group('attach_addr')
-                url = wikiutil.url_unquote(attachment, want_unicode=True)
+                url = wikiutil.url_unquote(attachment)
                 text = self.get_text(node)
                 return ''.join([
                         self.formatter.attachment_link(1, url),
@@ -325,13 +323,13 @@ class Emitter:
     def image_emit(self, node):
         target = node.content
         text = self.get_text(node)
-        m = self.addr_re.match(target)
+        m = self.rules.addr_re.match(target)
         if m:
             if m.group('page_name'):
                 # inserted anchors
-                url = wikiutil.url_unquote(target, want_unicode=True)
+                url = wikiutil.url_unquote(target)
                 if target.startswith('#'):
-                    return self.formatter.rawHTML(u'<a name="%s"></a>' % url[1:])
+                    return self.formatter.anchordef(url[1:])
                 # default to images
                 return self.formatter.attachment_image(
                     url, alt=text, html_class='image')
@@ -339,38 +337,39 @@ class Emitter:
                 # external link
                 address = m.group('extern_addr')
                 proto = m.group('extern_proto')
-                url = wikiutil.url_unquote(address, want_unicode=True)
+                url = wikiutil.url_unquote(address)
                 return self.formatter.image(
                     src=url, alt=text, html_class='external_image')
             elif m.group('attach_scheme'):
                 # link to an attachment
                 scheme = m.group('attach_scheme')
                 attachment = m.group('attach_addr')
-                url = wikiutil.url_unquote(attachment, want_unicode=True)
+                url = wikiutil.url_unquote(attachment)
                 if scheme == 'image':
                     return self.formatter.attachment_image(
                         url, alt=text, html_class='image')
                 elif scheme == 'drawing':
-                    return self.formatter.attachment_drawing(url, text)
+                    url = wikiutil.drawing2fname(url)
+                    return self.formatter.attachment_drawing(url, text, alt=text)
                 else:
                     pass
             elif m.group('inter_wiki'):
                 # interwiki link
                 pass
 #        return "".join(["{{", self.formatter.text(target), "}}"])
-        url = wikiutil.url_unquote(node.content, want_unicode=True)
+        url = wikiutil.url_unquote(node.content)
         return self.formatter.attachment_inlined(url, text)
 
 # Not used
 #    def drawing_emit(self, node):
-#        url = wikiutil.url_unquote(node.content, want_unicode=True)
+#        url = wikiutil.url_unquote(node.content)
 #        text = self.get_text(node)
 #        return self.formatter.attachment_drawing(url, text)
 
 # Not used
 #    def figure_emit(self, node):
 #        text = self.get_text(node)
-#        url = wikiutil.url_unquote(node.content, want_unicode=True)
+#        url = wikiutil.url_unquote(node.content)
 #        return ''.join([
 #            self.formatter.rawHTML('<div class="figure">'),
 #            self.get_image(url, text), self.emit_children(node),
