@@ -101,7 +101,7 @@ Detailed Instructions:
 General syntax: moin [options] export dump [dump-options]
 
 [options] usually should be:
-    --config-dir=/path/to/my/cfg/ --wiki-url=wiki.example.org/
+    --config-dir=/path/to/my/cfg/ --wiki-url=http://wiki.example.org/
 
 [dump-options] see below:
     0. You must run this script as owner of the wiki files, usually this is the
@@ -199,9 +199,9 @@ General syntax: moin [options] export dump [dump-options]
                     pagehtml = request.redirectedOutput(page.send_page, count_hit=0, content_only=1)
                 except:
                     errcnt = errcnt + 1
-                    print >>sys.stderr, "*** Caught exception while writing page!"
-                    print >>errlog, "~" * 78
-                    print >>errlog, file # page filename
+                    print >> sys.stderr, "*** Caught exception while writing page!"
+                    print >> errlog, "~" * 78
+                    print >> errlog, file # page filename
                     import traceback
                     traceback.print_exc(None, errlog)
             finally:
@@ -230,4 +230,5 @@ General syntax: moin [options] export dump [dump-options]
 
         errlog.close()
         if errcnt:
-            print >>sys.stderr, "*** %d error(s) occurred, see '%s'!" % (errcnt, errfile)
+            print >> sys.stderr, "*** %d error(s) occurred, see '%s'!" % (errcnt, errfile)
+
