@@ -19,9 +19,6 @@
     @license: GNU GPL, see COPYING for details.
 """
 
-# add names here to hide them in the cgitb traceback
-unsafe_names = ("id", "key", "val", "user_data", "enc_password", "recoverpass_key")
-
 import os, time, codecs, base64
 
 from MoinMoin.support.python_compatibility import hash_new, hmac_new
@@ -197,7 +194,7 @@ def isValidName(request, name):
     @param name: user name, unicode
     """
     normalized = normalizeName(name)
-    return (name == normalized) and not wikiutil.isGroupPage(request, name)
+    return (name == normalized) and not wikiutil.isGroupPage(name, request.cfg)
 
 
 def encodeList(items):
@@ -332,7 +329,6 @@ class User:
 
         # attrs not saved to profile
         self._request = request
-        self._trail = []
 
         # we got an already authenticated username:
         check_password = None
@@ -478,9 +474,6 @@ class User:
         if -24 <= self.tz_offset and self.tz_offset <= 24:
             self.tz_offset = self.tz_offset * 3600
 
-        # clear trail
-        self._trail = []
-
         if not self.disabled:
             self.valid = 1
 
@@ -499,7 +492,7 @@ class User:
         This is a private method and should not be used by clients.
 
         @param data: dict with user data (from storage)
-        @param password: password to verify
+        @param password: password to verify [unicode]
         @rtype: 2 tuple (bool, bool)
         @return: password is valid, enc_password changed
         """
@@ -513,19 +506,17 @@ class User:
         if not password:
             return False, False
 
-        password = password.encode('utf-8')
-
         if epwd[:5] == '{SHA}':
-            enc = '{SHA}' + base64.encodestring(hash_new('sha1', password).digest()).rstrip()
+            enc = '{SHA}' + base64.encodestring(hash_new('sha1', password.encode('utf-8')).digest()).rstrip()
             if epwd == enc:
-                data['enc_password'] = encodePassword(password)
+                data['enc_password'] = encodePassword(password) # upgrade to SSHA
                 return True, True
             return False, False
 
         if epwd[:6] == '{SSHA}':
             data = base64.decodestring(epwd[6:])
             salt = data[20:]
-            hash = hash_new('sha1', password)
+            hash = hash_new('sha1', password.encode('utf-8'))
             hash.update(salt)
             return hash.digest() == data[:20], False
 
@@ -570,8 +561,9 @@ class User:
             elif isinstance(value, dict):
                 key += '{}'
                 value = encodeDict(value)
-            line = u"%s=%s\n" % (key, unicode(value))
-            data.write(line)
+            line = u"%s=%s" % (key, unicode(value))
+            line = line.replace('\n', ' ').replace('\r', ' ') # no lineseps
+            data.write(line + '\n')
         data.close()
 
         arena = 'user'
@@ -880,7 +872,7 @@ class User:
     # Trail
 
     def _wantTrail(self):
-        return (not self.valid and self._request.session.is_stored  # anon session
+        return (not self.valid and self._request.cfg.cookie_lifetime[0]  # anon sessions enabled
                 or self.valid and (self.show_page_trail or self.remember_last_visit))  # logged-in session
 
     def addTrail(self, page):
@@ -889,9 +881,6 @@ class User:
         @param page: the page (object) to add to the trail
         """
         if self._wantTrail():
-            # load trail if not known
-            self.getTrail()
-
             pagename = page.page_name
             # Add only existing pages that the user may read
             if not (page.exists() and self._request.user.may.read(pagename)):
@@ -901,20 +890,22 @@ class User:
             if self._cfg.interwikiname:
                 pagename = self._interWikiName(pagename)
 
+            trail = self._request.session.get('trail', [])
+            trail_current = trail[:]
+
             # Don't append tail to trail ;)
-            if self._trail and self._trail[-1] == pagename:
+            if trail and trail[-1] == pagename:
                 return
 
             # Append new page, limiting the length
-            self._trail = [p for p in self._trail if p != pagename]
-            self._trail = self._trail[-(self._cfg.trail_size-1):]
-            self._trail.append(pagename)
-            self.saveTrail()
-
-    def saveTrail(self):
-        """ Save trail into session """
-        if not self._request.session.is_new:
-            self._request.session['trail'] = self._trail
+            trail = [p for p in trail if p != pagename]
+            pagename_stripped = pagename.strip()
+            if pagename_stripped:
+                trail.append(pagename_stripped)
+            trail = trail[-self._cfg.trail_size:]
+            if trail != trail_current:
+                # we only modify the session if we have something different:
+                self._request.session['trail'] = trail
 
     def getTrail(self):
         """ Return list of recently visited pages.
@@ -922,13 +913,11 @@ class User:
         @rtype: list
         @return: pages in trail
         """
-        if not self._trail and self._wantTrail():
+        if self._wantTrail():
             trail = self._request.session.get('trail', [])
-            trail = [t.strip() for t in trail]
-            trail = [t for t in trail if t]
-            self._trail = trail[-self._cfg.trail_size:]
-
-        return self._trail
+        else:
+            trail = []
+        return trail
 
     # -----------------------------------------------------------------
     # Other
@@ -939,13 +928,15 @@ class User:
 
     def isSuperUser(self):
         """ Check if this user is superuser """
+        if not self.valid:
+            return False
         request = self._request
-        if request.cfg.DesktopEdition and request.remote_addr == '127.0.0.1' and request.user and request.user.valid:
+        if request.cfg.DesktopEdition and request.remote_addr == '127.0.0.1':
             # the DesktopEdition gives any local user superuser powers
             return True
         superusers = request.cfg.superuser
         assert isinstance(superusers, (list, tuple))
-        return self.valid and self.name and self.name in superusers
+        return self.name and self.name in superusers
 
     def host(self):
         """ Return user host """
@@ -1026,11 +1017,11 @@ Login Name: %s
 
 Password recovery token: %s
 
-Password reset URL: %s/?action=recoverpass&name=%s&token=%s
+Password reset URL: %s?action=recoverpass&name=%s&token=%s
 """) % (
                         self.name,
                         tok,
-                        self._request.getBaseURL(),
+                        self._request.url_root,
                         url_quote_plus(self.name),
                         tok, )
 
