@@ -59,7 +59,7 @@ class Settings(UserPrefBase):
         _ = self.request.getText
         request = self.request
 
-        openid_id = request.form.get('openid_identifier', [''])[0]
+        openid_id = request.form.get('openid_identifier', '')
         if not openid_id:
             return 'error', _("No OpenID given.")
 
@@ -83,7 +83,7 @@ class Settings(UserPrefBase):
                     'handler': 'oid',
                     'oid.return': '1'}
             return_to = request.getQualifiedURL(request.page.url(request, qstr))
-            trust_root = request.getBaseURL()
+            trust_root = request.url_root
             if oidreq.shouldSendRedirect():
                 redirect_url = oidreq.redirectURL(trust_root, return_to)
                 request.http_redirect(redirect_url)
@@ -101,7 +101,7 @@ class Settings(UserPrefBase):
                                         MoinOpenIDStore(request))
         query = {}
         for key in request.form:
-            query[key] = request.form[key][0]
+            query[key] = request.form[key]
         qstr = {'action': 'userprefs',
                 'handler': 'oid',
                 'oid.return': '1'}
@@ -133,13 +133,16 @@ class Settings(UserPrefBase):
         _ = self._
         form = self.request.form
 
-        if form.has_key('oid.return'):
+        if self.request.values.has_key('oid.return'):
             return self._handle_oidreturn()
 
         if form.has_key('cancel'):
             return
 
-        if self.request.request_method != 'POST':
+        if self.request.method != 'POST':
+            return
+
+        if not wikiutil.checkTicket(self.request, form.get('ticket', '')):
             return
 
         if form.has_key('remove'):
@@ -148,15 +151,13 @@ class Settings(UserPrefBase):
         if form.has_key('add'):
             return self._handle_add()
 
-        return
-
     def _make_form(self):
-        sn = self.request.getScriptname()
-        pi = self.request.getPathinfo()
-        action = u"%s%s" % (sn, pi)
+        action = "%s%s" % (self.request.script_root, self.request.path)
         _form = html.FORM(action=action)
         _form.append(html.INPUT(type="hidden", name="action", value="userprefs"))
         _form.append(html.INPUT(type="hidden", name="handler", value="oid"))
+        ticket = wikiutil.createTicket(self.request)
+        _form.append(html.INPUT(type="hidden", name="ticket", value=ticket))
         return _form
 
     def _make_row(self, label, cell, **kw):
